@@ -25,7 +25,7 @@ use Encode 'encode_utf8';
 
 use experimental 'postderef';
 
-our $VERSION = "v0.0.19";
+our $VERSION = "v0.0.20";
 
 use constant {
     DEFAULT_CLEANUP_INTERVAL          => 5,
@@ -89,7 +89,9 @@ sub register ($self, $app, $config) {
             op_share(),
         );
 
-        my $graceful_stop_o = rx_from_event(Mojo::IOLoop->singleton, 'finish');
+        my $graceful_stop_o = rx_from_event(Mojo::IOLoop->singleton, 'finish')->pipe(
+            op_share(),
+        );
 
         # on graceful stop, boot all clients
         for my $i (1 .. 10) {
@@ -200,6 +202,7 @@ sub register ($self, $app, $config) {
                 return rx_timer($cleanup_interval * 0.5 + rand($cleanup_interval));
             })->pipe(
                 op_repeat(),
+                op_take_until($graceful_stop_o),
             )->subscribe(async sub {
                 my $t0 = Time::HiRes::time();
                 my $cleanup_state = $app->$bs_prefix->get_state($CLEANUP_CHANNEL);
@@ -230,7 +233,9 @@ sub register ($self, $app, $config) {
         # Remove this code when this issue has been addressed:
         #  https://github.com/mojolicious/mojo-pg/issues/77
         $app->$bs_prefix->pubsub->listen('dummy_notification' => sub {});
-        rx_timer(rand(0.5), 0.5)->subscribe(sub {
+        rx_timer(rand(0.5), 0.5)->pipe(
+            op_take_until($graceful_stop_o),
+        )->subscribe(sub {
             $app->$bs_prefix->pubsub->notify('dummy_notification' => '{}');
         });
     });
@@ -619,12 +624,8 @@ sub register ($self, $app, $config) {
 
                 # call cleanup sub
                 if (my $cleanup_sub = BoardStreams::Registry->get_cleanup($channel_name)) {
-                    $txn->lock_state($channel_name, sub ($state) {
-                        my $alive_workers = $c->$bs_prefix->alive_workers;
-                        my $new_state = $cleanup_sub->($channel_name, $state, $alive_workers);
-
-                        return undef, $new_state;
-                    }, no_ban => 1);
+                    my $alive_workers = $c->$bs_prefix->alive_workers;
+                    $cleanup_sub->($channel_name, $txn, $alive_workers);
                 }
 
                 $db->update('guards',
